import os
import re
import logging
import subprocess

#WE PROBABLY DON'T NEED THIS
# device info - describes SCSI device which was previously configured for
# MD RAID
# this function works only for scsi block devices
# attributes:
#    wwid - SCSI wwn of the device
#    path - filesystem path to the device
#    md_uuid - MD RAID uuid
#    md_num_devs - number of devices of RAID this dev belong to
#    md_level - RAID level of RAID this dev belong to
#def get_md_scsi_device_generic_info(path):
#
#    device_info = {}
#
#    stdout = subprocess.check_output(['/lib/udev/scsi_id', '--whitelisted', path])
#
#    lines = stdout.splitlines()
#
#    lines = stdout.splitlines()
#    if len(lines) != 1:
#        logging.warning('bullshit nebo nema')
#
#    if len(lines[0]) < 5:
#        logging.warning('weird uuid')
#
#    device_info['wwn'] = lines[0]
#    #device_info['path'] = path
#
#
#    cmd = '/sbin/mdadm --examine --export {}'.format(path)
#    outlines = subprocess.check_output(cmd,shell=True)
#
#    mdadm_values = {}
#    for line in outlines.splitlines():
#        k,v = line.split('=')
#        mdadm_values[k] = v
#
#
#    device_info['md_uuid'] = mdadm_values['MD_UUID']
#    device_info['md_num_devs']  = mdadm_values['MD_DEVICES']
#    device_info['md_level'] = mdadm_values['MD_LEVEL']
#
#    return device_info


def get_mdadm_detail(dev_path):

    out = subprocess.check_output(['mdadm', '--detail', dev_path, '-Y'])
    logging.info("Fetching MDADM details for dev_path: %s.", dev_path)

    mdadm_values = {}
    for line in out.splitlines():
        k, v = line.split('=')
        mdadm_values[k] = v

    md_detail = {}
    md_detail['uuid'] = mdadm_values['MD_UUID']
    md_detail['name'] = mdadm_values['MD_NAME']
    md_detail['level'] = mdadm_values['MD_LEVEL']
    md_detail['num_devices'] = mdadm_values['MD_DEVICES']
    #md_detail['dev_name'] = mdadm_values['MD_DEVNAME']
    #md_detail['dev_path'] = dev_path

    d = [v for k, v in mdadm_values.items() if re.search('MD_DEVICE.*_DEV', k)]
    md_detail['devices'] = d

    return md_detail


# return system-wide info -
#
# !!  local to the caller !! - careful
# with /dev/path this call can (and should) return dev_path but
# (as it is local runtime) but it has to be removed when storing
# to consul - we want to store "locality-agnostic" info only
def get_md_local_runtime_info_by_uuid():

    md_arrays = {}
    #we should check output and not relly on retval
    #because of permission denied on some files
    out = subprocess.check_output(['find', '/sys/devices', '-name', 'array_state'])
    logging.info("Fetching list of available MD devices from a system.")


    for line in out.splitlines():
        md_state = subprocess.check_output(['cat', line]).splitlines()[0]
        if md_state != 'active' and md_state != 'clean':
            continue

        tmp = os.path.split(line)[0]
        sys_root = os.path.split(tmp)[0]
        sys_dev_name = os.path.split(sys_root)[1]

        dev_path = '/dev/{}'.format(sys_dev_name)

        md_detail = get_mdadm_detail(dev_path)
        md_detail['state'] = md_state
        md_detail['dev_path'] = dev_path

        md_arrays[md_detail['uuid']] = md_detail

    return md_arrays


# FIXME: Would be fine to restore md device to original state after failure.
def mdadm_assemble_by_uuid(md_uuid, md_dev_path):

    #check_output raises exception if ret code non-zero
    logging.info("Trying to assemble raid with uuid: %s and md_dev_path: %s.", md_uuid, md_dev_path)
    subprocess.check_output(['mdadm', '-A', '-u', md_uuid, md_dev_path])




import os
import sys
import subprocess
import re

import logging

from retrying import retry

# returns system-wide info
def get_lvm_runtime_info_by_vg():

    out = subprocess.check_output(['pvdisplay', '-c'])
    logging.info("Fetching LVM information from a system.")

    pvs = {}
    for line in out.splitlines():
        parts = line.split(':')

        # If possible get existing record
        record = pvs.get(parts[1], {})

        pv_paths = record.get('pv_path', [])
        pv_paths.append(parts[0])

        record['pv_path'] = pv_paths
        record['vg_name'] = parts[1]
        record['uuid'] = parts[11]
        pvs[parts[1]] = record

    return pvs

def get_mounted_filesystems_by_mountpoint():

    out = subprocess.check_output(['/bin/cat', '/proc/mounts'])
    logging.info("Fetching mounted filesystem info from a system.")

    mounts = {}
    for line in out.splitlines():
        record = {}
        parts = line.split(' ')
        if '/dev' in parts[0]:
            record['dev_path'] = parts[0]
            record['mountpoint'] = parts[1]
            record['fs_type'] = parts[2]
            mounts[parts[1]] = record

    return mounts

def clear_env(clear_env_vars=[]):

    for i in clear_env_vars:
        if i in os.environ:
            del os.environ[i]


#!/usr/bin/env python

from distutils.core import setup

setup(name='os_nova_ha_utils',
      version='1.2.3',
      description='Python Openstack utils package',
      author='Premysl Kouril',
      author_email='pkouril@cra.cz',
      url='http://cra.cz',
      packages=['nova_ha_utils'],
      license='MIT',
      long_description=open('README.md').read(),
      install_requires=[
          'retrying',
          'mdstat'
      ],
      setup_requires=['retrying', 'mdstat'],
      tests_require=['pytest', 'mock'],
     )



import os
import subprocess
import logging

from retrying import retry
from mdadm import get_mdadm_detail

class BlockDeviceErrors(BaseException):
    """ Block Device exceptions """

# We make it failsafe because scsu_id queries the actual device via
# potentially undereliable SAS network
@retry(wait_fixed=2000, stop_max_attempt_number=5)
def get_scsi_device_wwn(path):
    logging.info("Getting disk WWN info from device: %s.", path)

    stdout = subprocess.check_output(['/lib/udev/scsi_id', '--whitelisted', path])

    lines = stdout.splitlines()

    if len(lines) != 1:
        logging.debug("Can't find device WWN in list: %s", lines)
        raise BlockDeviceErrors('SCSI WWN ERROR')

    if len(lines[0]) < 5:
        logging.debug("WWN seems to be in a wrong format: %s", lines[0])
        raise BlockDeviceErrors('SCSI WWN ERROR')

    return lines[0]

def get_scsi_device_map_by_wwn():
    return _get_device_map_by_wwn('scsi')


def get_multipath_device_map_by_wwn():
    return _get_device_map_by_wwn('dm')


def _get_device_map_by_wwn(dev_type):
    if dev_type == 'dm':
        scsi_devs = []
        for dev in os.listdir('/sys/block'):
            uuid_path = '/sys/block/{}/dm/uuid'.format(dev)
            if os.path.exists(uuid_path):
                if 'mpath' in open(uuid_path).read():
                    scsi_devs.append('/dev/{}'.format(dev))

    elif dev_type == 'scsi':
        scsi_devs = ['/dev/{}'.format(x) for x in os.listdir('/sys/block') if
                     os.path.exists('/sys/block/{}/device/scsi_device'.format(x))]
    else:
        scsi_devs = []

    logging.info("Creating device map for dev_type: %s and block_devices: %s", dev_type, scsi_devs)

    dev_map = {}
    for dev_path in scsi_devs:
        wwn = get_scsi_device_wwn(dev_path)
        dev_map[wwn] = dev_path

    return dev_map


def get_block_device_basic_info(path):

    info = {}
    dev_name = os.path.split(path)[1]

    logging.info("Getting block device info for a device: %s", path)

    # If we have received multipath device use first path as a device name
    if os.path.exists('/sys/block/{}/dm/uuid'.format(dev_name)):
        dev_name = os.listdir('/sys/block/{}/slaves'.format(dev_name))[0]

    if os.path.exists('/sys/block/{}/device/scsi_device'.format(dev_name)):
        info['uuid'] = get_scsi_device_wwn('/dev/{}'.format(dev_name))
        info['type'] = 'scsi'
    elif os.path.exists('/sys/block/{}/md/array_state'.format(dev_name)):
        md_detail = get_mdadm_detail('/dev/{}'.format(dev_name))
        info['uuid'] = md_detail['uuid']
        info['type'] = 'md'
    else:
        raise BlockDeviceErrors("unsupported device type")

    return info
